﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;
using DG.Tweening;

namespace OldScripts
{
    public enum GameMode { zombieSurvival, deathMatch, none }

    public class OldGameManager : MonoBehaviour
    {
        static public int numberOfTeams = 1;
        static public int numberOfPlayers = 2;
        static public GameMode initialGameMode = GameMode.none;

        public GameMode gameMode = GameMode.zombieSurvival;
        static OldGameManager _instance;
        public static OldGameManager Instance
        {
            get
            {
                if (_instance == null)
                    _instance = FindObjectOfType<OldGameManager>();
                return _instance;
            }
        }

        static public Dictionary<Player, PlayerScript> players = new Dictionary<Player, PlayerScript>();
        static public Dictionary<Team, List<Player>> teams = new Dictionary<Team, List<Player>>();
        static public Dictionary<Team, bool> teamsLiving = new Dictionary<Team, bool>();


        public List<Transform> enemySpawnPoints = new List<Transform>();
        [Tooltip("Number of spawnpoints must be equal to the number of players. Order of spawnpoints will be considered during spawning.")]
        public List<Transform> playerSpawnPoints = new List<Transform>();
        public List<GameObject> playerPrefabs = new List<GameObject>();
        List<Transform> playerRespawnPoints = new List<Transform>();
        public Transform playerRespawnRoot;

        public GameObject endScreen;
        public GameObject HUD;

        public Material teamNoneMaterial, teamOneMaterial, teamTwoMaterial, teamThreeMaterial, teamFourMaterial, teamFiveMaterial;

        public Sprite ballCloud, cannonCloud, rumCloud;

        public float timeBetweenWaves = 5f;
        bool betweenWaves = true;

        public GameObject NPC;
        int currentWave = 0;
        int wavefix = 0;

        public int spawned = 0;
        public int npcDied = 0;
        float time = 0f;

        float spawnTime;

        bool gameOver = false;
        public bool GameOver
        {
            get { return gameOver; }
        }

        List<GameObject> npcs = new List<GameObject>();

        void Awake()
        {
            DOTween.SetTweensCapacity(500, 50);
            gameMode = initialGameMode;
            List<PlayerScript> initialPlayers = new List<PlayerScript>();
            for (int i = 0; i < numberOfPlayers; i++)
            {
                GameObject go = Instantiate(playerPrefabs[i], playerSpawnPoints[i].position, playerSpawnPoints[i].rotation, null);
                initialPlayers.Add(go.GetComponent<PlayerScript>());
                initialPlayers[i].mySpawnPoint = playerSpawnPoints[i];
            }

            for (int i = 0; i < initialPlayers.Count; i++)
            {
                initialPlayers[i].playerNumber = (Player)i;
                if (numberOfTeams == 1) initialPlayers[i].team = Team.none;
                else if (numberOfTeams == 2)
                {
                    if (i < initialPlayers.Count * 0.5f) initialPlayers[i].team = Team.one;
                    else initialPlayers[i].team = Team.two;
                }
                else if (numberOfTeams == 3)
                {
                    if (i > (initialPlayers.Count / 3f) * 2f) initialPlayers[i].team = Team.three;
                    else if (i > (initialPlayers.Count / 3f)) initialPlayers[i].team = Team.two;
                    else initialPlayers[i].team = Team.one;
                }
                else if (numberOfTeams == 4)
                {
                    if (i > (initialPlayers.Count * 0.25f) * 3f) initialPlayers[i].team = Team.four;
                    else if (i > (initialPlayers.Count * 0.25f) * 2f) initialPlayers[i].team = Team.three;
                    else if (i > initialPlayers.Count * 0.25f) initialPlayers[i].team = Team.two;
                    else initialPlayers[i].team = Team.one;
                }
                initialPlayers[i].UpdateTeamColor();
            }

            for (int i = 0; i < playerRespawnRoot.childCount; ++i)
            {
                playerRespawnPoints.Add(playerRespawnRoot.GetChild(i));
            }

            RefreshPlayers();
        }


        void Update()
        {
            if (players.Count <= 0)
            {
                RefreshPlayers();
            }
            if (Input.GetKeyDown(KeyCode.Escape) && gameMode != GameMode.none) SceneManager.LoadScene("Menu");
            if (!gameOver)
            {
                if (gameMode == GameMode.deathMatch)
                {
                    //check witch teams are dead
                    foreach (KeyValuePair<Team, List<Player>> pair in teams)
                    {
                        if (teamsLiving[pair.Key] == false) continue;
                        bool someoneAlive = false;
                        foreach (Player p in pair.Value)
                        {
                            if (!GetPlayerScript(p).dead)
                            {
                                someoneAlive = true;
                                break;
                            }
                        }
                        if (!someoneAlive)
                        {
                            teamsLiving[pair.Key] = false;
                            break;
                        }
                    }
                    Team tempTeam = Team.invalid;
                    int livingTeamsCount = 0;
                    foreach (KeyValuePair<Team, bool> alive in teamsLiving)
                    {
                        if (alive.Value == true)
                        {
                            tempTeam = alive.Key;
                            livingTeamsCount++;
                        }
                    }
                    if (tempTeam != Team.invalid && livingTeamsCount == 1) EndGame(tempTeam);
                }
                else if (gameMode == GameMode.none)
                {
                    if (Input.GetKeyDown(KeyCode.Escape))
                    {
                        Application.Quit();
                    }
                    //check witch teams are dead
                    foreach (KeyValuePair<Team, List<Player>> pair in teams)
                    {
                        if (teamsLiving[pair.Key] == false) continue;
                        bool someoneAlive = false;
                        foreach (Player p in pair.Value)
                        {
                            if (!GetPlayerScript(p).dead)
                            {
                                someoneAlive = true;
                                break;
                            }
                        }
                        if (!someoneAlive)
                        {
                            teamsLiving[pair.Key] = false;
                            break;
                        }
                    }
                    Team tempTeam = Team.invalid;
                    int livingTeamsCount = 0;
                    foreach (KeyValuePair<Team, bool> alive in teamsLiving)
                    {
                        if (alive.Value == true)
                        {
                            tempTeam = alive.Key;
                            livingTeamsCount++;
                        }
                    }
                    if (livingTeamsCount == 0)
                        EndGame(tempTeam);
                }
            }
            else
            {
                if (gameMode == GameMode.none)
                {
                    if (Input.GetKeyDown(KeyCode.Space))
                    {
                        Restart();
                        SceneManager.LoadScene(SceneManager.GetActiveScene().name);
                    }
                    if (Input.GetKeyDown(KeyCode.Escape))
                    {
                        Application.Quit();
                    }
                }
            }
        }

        public void EndGame(Team team)
        {
            gameOver = true;
            ShowEndScreen(team);
        }

        void ShowEndScreen(Team team)
        {
            if (gameMode == GameMode.deathMatch)
            {
                // loseText.text = "Team " + team.ToString() + " have won!";
            }
            HUD.SetActive(false);
            endScreen.SetActive(true);
        }


        public void Restart()
        {
            gameOver = false;
            endScreen.SetActive(false);
            HUD.SetActive(true);
            foreach (PlayerScript ps in players.Values)
            {
                ps.Reset();
            }
            currentWave = 0;
            wavefix = 0;
            npcDied = 0;
            spawned = 0;
            npcDied = 0;
            time = 0f;
            RefreshPlayers();

            if (gameMode != GameMode.none) SceneManager.LoadScene("Menu");
        }

        public void SetTimeScale(float scale)
        {
            Time.timeScale = scale;
        }

        public Transform GetClosestRespawn(PlayerScript player)
        {
            Transform ret = player.transform;
            float distance = float.PositiveInfinity;
            float currentDistance = 0;
            foreach (Transform t in playerRespawnPoints)
            {
                currentDistance = Vector3.Distance(t.position, player.transform.position);
                if (currentDistance < distance)
                {
                    ret = t;
                    distance = currentDistance;
                }
            }
            return ret;
        }

        void RefreshPlayers()
        {
            players.Clear();
            teams.Clear();
            teamsLiving.Clear();
            foreach (PlayerScript ps in FindObjectsOfType<PlayerScript>())
            {
                if (ps.gameObject.activeInHierarchy)
                {
                    if (!teams.ContainsKey(ps.team))
                    {
                        teams.Add(ps.team, new List<Player>());
                        teamsLiving.Add(ps.team, true);
                    }
                    teams[ps.team].Add(ps.playerNumber);
                    players.Add(ps.playerNumber, ps);
                }
            }
        }


        static public PlayerScript GetPlayerScript(Player playerNumber)
        {
            if (players.ContainsKey(playerNumber)) return players[playerNumber];
            return null;
        }

        static public PlayerScript GetClosestPlayer(Vector3 position)
        {
            PlayerScript ret = null;
            float distance = float.PositiveInfinity;
            foreach (PlayerScript ps in players.Values)
            {
                float dist = Vector3.Distance(position, ps.transform.position);
                if (dist < distance)
                {
                    ret = ps;
                    distance = dist;
                }
            }
            return ret;
        }

        static public PlayerScript GetClosestPlayerAlive(Vector3 position)
        {
            PlayerScript ret = null;
            float distance = float.PositiveInfinity;
            foreach (PlayerScript ps in players.Values)
            {
                if (ps.hp <= 0) continue;
                float dist = Vector3.Distance(position, ps.transform.position);
                if (dist < distance)
                {
                    ret = ps;
                    distance = dist;
                }
            }
            return ret;
        }



        void OnDestroy()
        {
            players.Clear();
        }
    }
}