﻿using DG.Tweening;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;



public class HermitController : MonoBehaviour
{
    public int hermitNo = -1;
    public int HermitNo
    {
        get
        {
            return hermitNo;
        }
        set
        {
            if(hermitNo == -1)
            {
                hermitNo = value;
            }
        }
    }

    public bool rdyToFight = false;
    public static int RdyTofightCount = 0;

    public int Health = 200;
    [Range(25, 75)]
    public int SpeedForce = 40;
    [Range(25,750)]
    public int EscapeJumpForce = 50;
    [Range(10,25)]
    public int RotationAngle = 15;
    [Range(0.01f, 1f)]
    public float RotationDuration = 0.1f;
    public bool Dead { get { return Health <= 0; } }

    public Image PressBImage;
    public Image ControlsImage;

    public string HermitName;

    public AudioSource softHit;
    public AudioSource hardHit;
    public AudioSource missHit;
    public AudioSource plasticArmorHit;
    public AudioSource armorPickUp;
    public AudioSource weaponPickUp;
    public AudioSource jump;

    private Animator thisAnimator;
    
    public Camera myCamera, UICamera;
    public Transform leftPivot;
    public Transform rightPivot;

    private PadAxisInputHandler input;
    private Rigidbody thisRigidbody;

    public Transform weaponPivot;
    public Transform shellPivot;

    public ObjectsInRange DmgBox;

    private List<PickupBase> availablePickups = new List<PickupBase>();

    public List<ParticleSystem> sandParticlesLeftLegs = new List<ParticleSystem>();
    public List<ParticleSystem> sandParticlesRightLegs = new List<ParticleSystem>();
    public List<ParticleSystem> onHitParticles = new List<ParticleSystem>();
    public List<ParticleSystem> brokenArmorParticles = new List<ParticleSystem>();

    private Weapon activeWeapon;
    private Armor activeArmor;
    private float plasticArmorTime;

    [SerializeField]
    private UIHealthBar HPBar;

    [SerializeField]
    private UIDefenseBars DefBar;

    [SerializeField]
    private GameObject UIObject;

    [SerializeField]
    private GameObject PickupInfoUI;

    public void SetInputForHermit(PadAxisInputHandler _input)
    {
        input = _input;
    }

    private void Awake()
    {
        HermitName = "John" + Random.Range(0, int.MaxValue)%100;
        myCamera.transform.SetParent(null);
        thisRigidbody = GetComponent<Rigidbody>();
        thisAnimator = GetComponentInChildren<Animator>();
        HPBar.Hide();
        DefBar.Hide();

        HermitNo = HermitsManager.Instance.HermitsCount;
        Init();
    }

    private void Init()
    {
        Health = Params.Me.StartPlayerHealth;
        HPBar.Init(Health);
        DefBar.Reinit(activeArmor);

        Assets.Scripts.Extensions.SetLayerRecursively(UIObject, 11 + hermitNo);
        UICamera.cullingMask = (1 << LayerMask.NameToLayer("UI" + (hermitNo + 1).ToString()));
    }

    List<LeftStickMove> LeftStickMoveList = new List<LeftStickMove>();

    int BonusRotateValue = 0;

    float LastTimeEscape = 0f;

    bool died = false;

    private void Update()
    {
        //if (input != null)
        //{
            if (Health <= 0 && died == false)
            {
                thisAnimator.SetTrigger("Die");
                died = true;
            }
            else if (died == false)
            {
                if (input.RBumberDown())
                {
                    BonusRotateValue = Mathf.Clamp(BonusRotateValue, 0, 4);
                    rotateRigidBodyAroundPointBy(thisRigidbody, rightPivot.position, Vector3.up, RotationAngle + RotationAngle * BonusRotateValue / 6);
                    Vector3 direction = transform.TransformPoint(Vector3.forward) - transform.position;
                    thisRigidbody.AddForce(direction * SpeedForce);
                    BonusRotateValue = Mathf.Clamp(BonusRotateValue + 1, 0, 4);
                    thisAnimator.SetTrigger("moveRight");
                    foreach (ParticleSystem system in sandParticlesRightLegs) system.Play();
                }
                if (input.LBumberDown())
                {
                    BonusRotateValue = Mathf.Clamp(BonusRotateValue, -3, 1);
                    rotateRigidBodyAroundPointBy(thisRigidbody, leftPivot.position, Vector3.up, -RotationAngle + RotationAngle * BonusRotateValue / 6);
                    Vector3 direction = transform.TransformPoint(Vector3.forward) - transform.position;
                    thisRigidbody.AddForce(direction * SpeedForce);
                    BonusRotateValue = Mathf.Clamp(BonusRotateValue - 1, -3, 1);
                    thisAnimator.SetTrigger("moveLeft");
                    foreach (ParticleSystem system in sandParticlesLeftLegs) system.Play();
                }
                if (input.LTrigger() > 0.8f && input.RTrigger() > 0.8f && (Time.time - LastTimeEscape > 2f))
                {
                    LastTimeEscape = Time.time;
                    Vector3 direction = transform.TransformPoint(new Vector3(0, 5, -5).normalized) - transform.position;
                    thisRigidbody.AddForce(direction * EscapeJumpForce);
                    jump.Play();
                    //thisAnimator.SetTrigger("Jump");
                }
                if (input.LStickHor() <= -0.75f && (LeftStickMoveList.Count == 0 || LeftStickMoveList[LeftStickMoveList.Count - 1] != LeftStickMove.Left))
                {
                    LeftStickMoveList.Add(LeftStickMove.Left);
                }
                if (input.LStickVer() <= -0.75f && (LeftStickMoveList.Count == 0 || LeftStickMoveList[LeftStickMoveList.Count - 1] != LeftStickMove.Up))
                {
                    LeftStickMoveList.Add(LeftStickMove.Up);
                }
                if (input.LStickVer() >= 0.75f && (LeftStickMoveList.Count == 0 || LeftStickMoveList[LeftStickMoveList.Count - 1] != LeftStickMove.Down))
                {
                    LeftStickMoveList.Add(LeftStickMove.Down);
                }
                if (input.LStickVer() == 0 && input.LStickHor() == 0)
                {
                    int count = LeftStickMoveList.Count;
                    if (count >= 3)
                    {
                        if (LeftStickMoveList[count - 1] == LeftStickMove.Up &&
                            LeftStickMoveList[count - 2] == LeftStickMove.Left &&
                            LeftStickMoveList[count - 3] == LeftStickMove.Down)
                        {
                            Debug.Log("POWER HIT");
                            TryToHit(30, "hardhit");
                        }
                    }
                    else if (count >= 2)
                    {
                        if (LeftStickMoveList[count - 1] == LeftStickMove.Up &&
                            LeftStickMoveList[count - 2] == LeftStickMove.Down)
                        {
                            Debug.Log("FORCE HIT");
                            TryToHit(20, "FastForward");
                        }
                    }
                    else if (count >= 1)
                    {
                        if (LeftStickMoveList[count - 1] == LeftStickMove.Up)
                        {
                            Debug.Log("FRONT HIT");
                            TryToHit(10, "FastForward");
                        }
                        else if (LeftStickMoveList[count - 1] == LeftStickMove.Left)
                        {
                            Debug.Log("LEFT HIT");
                            TryToHit(10, "FastForward");
                        }
                        else if (LeftStickMoveList[count - 1] == LeftStickMove.Down)
                        {
                            Debug.Log("BLOCK");
                        }
                    }
                    LeftStickMoveList.Clear();
                }
                if (this.activeArmor != null)
                {
                    if (this.activeArmor.armorType == ArmorType.plastic)
                    {
                        this.plasticArmorTime += Time.deltaTime;
                        if (this.plasticArmorTime > 1)
                        {
                            this.plasticArmorTime = 0;
                            this.Health -= 3;
                            HPBar.Refresh(Health, activeArmor);

                        }

                    }
                }
            }
        //}
    }

    void rotateRigidBodyAroundPointBy(Rigidbody rb, Vector3 origin, Vector3 axis, float angle)
    {
        Quaternion q = Quaternion.AngleAxis(angle, axis);
        rb.DORotate((rb.transform.rotation * q).eulerAngles, RotationDuration);
    }

    public void ResetCamera()
    {
        switch (hermitNo)
        {
            case 0:
                {
                    myCamera.rect = new Rect(0f, 0.5f, 0.5f, 0.5f);
                    UICamera.rect = new Rect(0f, 0.5f, 0.5f, 0.5f);
                }
                break;
            case 1:
                {
                    myCamera.rect = new Rect(0.5f, 0.5f, 0.5f, 0.5f);
                    UICamera.rect = new Rect(0.5f, 0.5f, 0.5f, 0.5f);
                }
                break;
            case 2:
                {
                    myCamera.rect = new Rect(0f, 0f, 0.5f, 0.5f);
                    UICamera.rect = new Rect(0f, 0f, 0.5f, 0.5f);
                }
                break;
            case 3:
                {
                    myCamera.rect = new Rect(0.5f, 0f, 0.5f, 0.5f);
                    UICamera.rect = new Rect(0.5f, 0f, 0.5f, 0.5f);
                }
                break;
        }
    }
    
    enum LeftStickMove
    {
        Up, Left, Down
    }

    public void OnTriggerStay(Collider other)
    {
        PickupBase item = other.GetComponent<PickupBase>();
        if (item != null)
        {
            if (!PickupInfoUI.activeSelf && item.isPickable)
                PickupInfoUI.SetActive(true);

            if (input.BKeyDown())
            {
                Debug.Log(item.pickupName + "TESTETSTSTSTSTS");
                //this.availablePickups.Add(item);

                this.PickupItemFromGround(item);
            }
        }
        else if (other.CompareTag("Respawn"))
        {
            transform.position = RaceGameManager.Instance.GetClosestRespawn(this).position;
        }

    }

    public void OnTriggerExit(Collider other)
    {
        PickupBase item = other.GetComponent<PickupBase>();
        if (item)
        {
            Debug.Log(other.GetComponent<PickupBase>().pickupName);
            //this.availablePickups.Remove(other.GetComponent<PickupBase>());
            if (PickupInfoUI.activeSelf)
                PickupInfoUI.SetActive(false);
        }

    }

    private PickupBase GetClosestPickup()
    {

        if (this.availablePickups.Count == 1)
        {
            return this.availablePickups[0];
        }
        else if (this.availablePickups.Count > 1)
        {
            PickupBase closestObject = null;
            float closestDistance = float.MaxValue;

            foreach (PickupBase item in  this.availablePickups)
            {
                float deltaX = this.transform.position.x - item.transform.position.x;
                float deltaY = this.transform.position.y - item.transform.position.y;
                float deltaZ = this.transform.position.z - item.transform.position.z;

                float distance = (float)System.Math.Sqrt(deltaX * deltaX + deltaY * deltaY + deltaZ * deltaZ);
                if (distance < closestDistance)
                {
                    closestDistance = distance;
                    closestObject = item;
                }
            }
            return closestObject;
        }

        return null;
    }




    private void PickupItemFromGround(PickupBase item)
    {
        if ( !item.isPickable)
        {
            return;
        }

        if (item is Armor)
        {
            if (this.activeArmor != null && this.activeArmor != item)
            {
                Debug.Log("REEMOVE ARMOR");

                if (this.activeArmor.armorType == ArmorType.plastic)
                {
                    foreach (ParticleSystem system in this.activeArmor.GetComponentsInChildren<ParticleSystem>())
                    {
                        system.Stop();
                    }

                    plasticArmorHit.Stop();
                }
                this.activeArmor.transform.SetParent(null);
                PickupManager.Instance.removePickup(this.activeArmor);
                activeArmor = null;

                //TODO: dodac animacje zdejmowania pancerza
                //TODO: uncomment this when dropping armor 
                DefBar.Reinit(this.activeArmor /*or null*/);
            }
            else if (this.activeArmor != item)
            {
                //TODO: dodac animacje zakładania  pancerza
                Debug.Log("PICKUP ARMOR");
                armorPickUp.Play();
                OnPickUpArmor(item as Armor);
            }

            HPBar.Refresh(Health, activeArmor);

        }
        else if (item is Weapon)
        {
            weaponPickUp.Play();
            if (this.activeWeapon != null)
                this.activeWeapon.transform.SetParent(null);

            Debug.Log("PICKUP WEAPON");

            this.activeWeapon = (Weapon)item;
            this.activeWeapon.SetRandomDurability();
            this.activeWeapon.isPickable = false;
            PickItemToPosition(item.gameObject, this.weaponPivot);
        }
    }

    public void TryToHit(int dmg,string hitype)
    {
        thisAnimator.SetTrigger(hitype);
        int weaponDamage = 0;
        if (this.activeWeapon != null)
        {
            weaponDamage = this.activeWeapon.weaponAttackValue;
        }

        List<GameObject> toHit = DmgBox.collidingObjects.FindAll(x => x != null).FindAll(x => x.GetComponent<HermitController>() != null);
        if (toHit.Count == 0)
        {
            //miss sound
            missHit.Play();
        }
        else
        {
            if(hitype == "FastForward")
            {
                //play fast hit
                softHit.Play();
            } else if (hitype == "hardhit")
            {
                //play strong hit
                hardHit.Play();
            }
                foreach (var item in toHit)
            {
                item.GetComponent<HermitController>().OnHit(dmg + weaponDamage, (item.transform.position - transform.position).normalized + Vector3.up);
            }
        }

        if (this.activeWeapon != null)
        {
            this.activeWeapon.DecreaseDurability(toHit.Count);
        }
    }

    public void OnHit(int Dmg)
    {
        OnHit(Dmg, Vector3.zero);
    }

    public void OnHit(int Dmg, Vector3 force)
    {
        if (this.activeArmor != null)
        {
            this.Health -= this.activeArmor.GetReducedHP(Dmg);
            if (this.activeArmor.GetRemainingHPValue() <= 0)
            {
                PickupManager.Instance.removePickup(this.activeArmor);
                foreach (ParticleSystem system in brokenArmorParticles) system.Play();
            }
        }
        else
        {
            Health -= Dmg;
        }

        thisRigidbody.AddForce(force * Dmg*2f);

        HPBar.Refresh(Health, activeArmor);
        DefBar.Refresh();
        if (onHitParticles.Count > 0) onHitParticles[Random.Range(0, onHitParticles.Count - 1)].Play();

        if (Dmg > 0)
            PickupManager.Instance.SpawnDmg(transform, Dmg);
    }

    public void OnPickUpArmor(Armor armor)
    {
        this.activeArmor = armor;
        this.activeArmor.isPickable = false;
        PickItemToPosition(armor.gameObject, this.shellPivot);
        if (this.activeArmor.armorType == ArmorType.plastic)
        {
            foreach(ParticleSystem system in this.activeArmor.GetComponentsInChildren<ParticleSystem>())
            {
                system.Play();
            }

            plasticArmorHit.Play();
        }



        DefBar.Reinit(armor);
    }

    private void PickItemToPosition(GameObject item, Transform newParent)
    {
        item.transform.SetParent(newParent);
        item.transform.localPosition = Vector3.zero;
        item.transform.localEulerAngles = Vector3.zero;
    }

    public void RemoveStartUI()
    {
        PressBImage.enabled = false;
        ControlsImage.enabled = false;
        HPBar.Show();
        DefBar.Show();
        rdyToFight = true;
        RdyTofightCount++;
    }

}
