﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

public class HermitsManager : MonoBehaviour
{
    private static HermitsManager instance;
    public static HermitsManager Instance
    {
        get
        {
            return instance; 
        }
        set
        {
            if(instance != null)
            {
                Debug.LogError("Too much HermitsManager, DESTROY !!!!");
                Destroy(value);
            }
            instance = value;
        }
    }

    public bool run = false;

    public GameObject HermitPrefab;
    Dictionary<int, PadAxisInputHandler> inputHandlers = new Dictionary<int, PadAxisInputHandler>();
    public Dictionary<int, GameObject> SpawnedHermits = new Dictionary<int, GameObject>();
    [Tooltip("Number of spawnpoints must be qual to number of players")]
    public List<Transform> spawnPoints = new List<Transform>();
    [Tooltip("Number of spawnpoints must be qual to number of players")]
    public List<Transform> initSpawnPoints = new List<Transform>();
    [Tooltip("Number of Colors must be qual to number of players")]
    public List<Color> playerColors = new List<Color>();
    public bool gameOn = false;
    public float pickupRefreshTime = 30f;
    public float gameTime = 180f;
    public float time = 0f;
    public float CurrentTime
    {
        get
        {
            return gameTime - time;
        }
    }
    float timeToRefresh = 30f;

    public int HermitsCount
    {
        get
        {
            return SpawnedHermits.Count;
        }
    }

    public Action OnHermitSpawn;
    public Action OnHermitDespawn;

    private void Awake()
    {
        Instance = this;
        foreach (var item in transform.GetComponents<PadAxisInputHandler>())
        {
            inputHandlers.Add(item.padNumber, item);
        }
    }

    private void Start()
    {
        foreach (var item in inputHandlers)
        {
            if (!SpawnedHermits.ContainsKey(item.Key))
            {
                SpawnedHermits.Add(item.Key, Instantiate(HermitPrefab, (initSpawnPoints.Count > 0 && initSpawnPoints.Count > item.Key - 1 ? initSpawnPoints[item.Key - 1].position : Vector3.up * 2f), Quaternion.identity));
                SpawnedHermits[item.Key].GetComponent<HermitController>().SetInputForHermit(item.Value);
                SpawnedHermits[item.Key].GetComponentInChildren<SkinnedMeshRenderer>().material.color = (playerColors.Count > 0 && playerColors.Count > item.Key - 1 ? playerColors[item.Key - 1] : Color.white);
                //SpawnedHermits[item.Key].SetActive(false);
                if(OnHermitSpawn != null)
                    OnHermitSpawn();
            }
        }
        ResetHermitsCamerasScreenPosition();
    }



    private void Update()
    {
        foreach (var item in inputHandlers)
        {
            if (item.Value.BKeyDown() && SpawnedHermits[item.Key].GetComponent<HermitController>().rdyToFight == false)
            {
                SpawnedHermits[item.Key].GetComponent<HermitController>().RemoveStartUI();
                SpawnedHermits[item.Key].transform.position = spawnPoints.Count > 0 && spawnPoints.Count > item.Key - 1 ? spawnPoints[item.Key - 1].position : Vector3.up * 2f;
                AudioManagement.Instance.PlayJoin();
                if (OnHermitSpawn != null)
                    OnHermitSpawn();
            }
        }


        if (HermitController.RdyTofightCount >= 2)
        {
            AudioManagement.Instance.SwitchToMusic();
            if (timeToRefresh >= pickupRefreshTime)
            {
                PickupManager.Instance.DisposeOfOldPickups();
                PickupManager.Instance.ComeWithNewPickups();
                timeToRefresh = 0f;
            }
            if(time>=gameTime || GetPlayersAlive()<=1)
            {
                gameOn = false;
                //time = 0f;
                timeToRefresh = 0f;
            }
            time += Time.deltaTime;
            timeToRefresh += Time.deltaTime;
        }
    }

    HermitController winningHermit = null;
    public int GetPlayersAlive()
    {
        int ret = 0;
        foreach (GameObject go in SpawnedHermits.Values)
        {
            HermitController hermit = go.GetComponent<HermitController>();
            if (hermit.Health > 0) ++ret;
            if (winningHermit == null || winningHermit.Health < hermit.Health) winningHermit = hermit;
        }
        if (ret != 1) winningHermit = null;
        return ret;
    }

    public void ResetHermitsCamerasScreenPosition()
    {
        foreach (var item in SpawnedHermits)
        {
            item.Value.GetComponent<HermitController>().ResetCamera();
        }
    }
}
