﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PadAxisInputHandler : MonoBehaviour
{
	[Header("Set only pad number")]
	public int padNumber;
	string padName;
	protected float pressTime;
	protected float safeDelta = 0.2f;
	protected bool leftDown, rightDown, leftDownMenu, rightDownMenu, downDownMenu, upDownMenu;
	protected bool leftFirst;


	string leftTrigger, rightTrigger;
    string leftStickX, leftStickY;
    string rightStickX, rightStickY;
    string keyA, keyB, keyX, keyY;
    string leftBumper, rightBumper;
#if UNITY_STANDALONE_WIN
    string dPadX, dPadY;
#endif
#if UNITY_STANDALONE_OSX
	string dPadUp, dPadDown, dPadLeft, dPadRight;
#endif

	// set up joysticks per platform (WIN/OSX)
	// Windows base: 8 axises, 6 buttons
	// Mac: -2 axises, +4 buttons
	void Awake()
	{
		if(padNumber != -1)
			InitInput();
		else
			MenuInputOnly();
	}

	protected void InitInput()
	{
		padName = "P" + padNumber;

		// left stick
		leftStickX = padName + "LeftStickX";
		leftStickY = padName + "LeftStickY";

		// right stick
#if UNITY_STANDALONE_WIN
		rightStickX = padName + "RightStickX";
		rightStickY = padName + "RightStickY";
#elif UNITY_STANDALONE_OSX
		rightStickX = padName + "RightStickXMac";
		rightStickY = padName + "RightStickYMac";
#endif

		// triggers
#if UNITY_STANDALONE_WIN
		leftTrigger = padName + "LeftTrigger";
		rightTrigger = padName + "RightTrigger";
#elif UNITY_STANDALONE_OSX
		leftTrigger = padName + "LeftTriggerMac";
		rightTrigger = padName + "RightTriggerMac";
#endif

		// d-pad
#if UNITY_STANDALONE_WIN
		dPadX = padName + "DPadX";
		dPadY = padName + "DPadY";
#elif UNITY_STANDALONE_OSX
		dPadUp = "joystick " + padNumber + " button 5";
		dPadDown = "joystick " + padNumber + " button 6";
		dPadLeft = "joystick " + padNumber + " button 7";
		dPadRight = "joystick " + padNumber + " button 8";
#endif

		// buttons
#if UNITY_STANDALONE_WIN
		keyA = "joystick " + padNumber + " button 0";
		keyB = "joystick " + padNumber + " button 1";
		keyX = "joystick " + padNumber + " button 2";
		keyY = "joystick " + padNumber + " button 3";
		leftBumper = "joystick " + padNumber + " button 4";
		rightBumper = "joystick " + padNumber + " button 5";
#elif UNITY_STANDALONE_OSX
		keyA = "joystick " + padNumber + " button 16";
		keyB = "joystick " + padNumber + " button 17";
		keyX = "joystick " + padNumber + " button 18";
		keyY = "joystick " + padNumber + " button 19";
		leftBumper = "joystick " + padNumber + " button 13";
		rightBumper = "joystick " + padNumber + " button 14";
#endif
	}

	protected void MenuInputOnly()
	{
		padName = "Any";

		// left stick
		leftStickX = padName + "LeftStickX";
		leftStickY = padName + "LeftStickY";

		// right stick
#if UNITY_STANDALONE_WIN
		rightStickX = padName + "RightStickX";
		rightStickY = padName + "RightStickY";
#elif UNITY_STANDALONE_OSX
		rightStickX = padName + "RightStickXMac";
		rightStickY = padName + "RightStickYMac";
#endif

		// triggers
#if UNITY_STANDALONE_WIN
		leftTrigger = padName + "LeftTrigger";
		rightTrigger = padName + "RightTrigger";
#elif UNITY_STANDALONE_OSX
		leftTrigger = padName + "LeftTriggerMac";
		rightTrigger = padName + "RightTriggerMac";
#endif

		// d-pad
#if UNITY_STANDALONE_WIN
		dPadX = padName + "DPadX";
		dPadY = padName + "DPadY";
#elif UNITY_STANDALONE_OSX
		dPadUp = "joystick button 5";
		dPadDown = "joystick button 6";
		dPadLeft = "joystick button 7";
		dPadRight = "joystick button 8";
#endif

		// buttons
#if UNITY_STANDALONE_WIN
		keyA = "joystick button 0";
		keyB = "joystick button 1";
		keyX = "joystick button 2";
		keyY = "joystick button 3";
		leftBumper = "joystick button 4";
		rightBumper = "joystick button 5";
#elif UNITY_STANDALONE_OSX
		keyA = "joystick button 16";
		keyB = "joystick button 17";
		keyX = "joystick button 18";
		keyY = "joystick button 19";
		leftBumper = "joystick button 13";
		rightBumper = "joystick button 14";
#endif
	}

	protected bool leftKeyDown()
	{
		if(Input.GetAxis(leftTrigger) > safeDelta
			|| Input.GetAxis(leftStickX) < -safeDelta
			|| Input.GetAxis(rightStickX) < -safeDelta
#if UNITY_STANDALONE_WIN
 || Input.GetAxis(dPadX) < -safeDelta
#elif UNITY_STANDALONE_OSX
			|| Input.GetKey(dPadLeft)
#elif UNITY_STANDALONE_LINUX
			
#endif
 || Input.GetKey(leftBumper))
		{
			return true;
		}
		return false;
	}

	protected bool leftKeyUp()
	{
		if(Input.GetAxis(leftTrigger) < safeDelta
			&& Input.GetAxis(leftStickX) > -safeDelta
			&& Input.GetAxis(rightStickX) > -safeDelta
#if UNITY_STANDALONE_WIN
 && Input.GetAxis(dPadX) > -safeDelta
#elif UNITY_STANDALONE_OSX
			&& !Input.GetKey(dPadLeft)
#elif UNITY_STANDALONE_LINUX
			
#endif
 && !Input.GetKey(leftBumper))
		{
			return true;
		}
		return false;
	}

    public bool BKeyDown()
    {
        if(Input.GetKeyDown(keyB))
        {
            return true;
        }
        return false;
    }

    public bool YKeyDown()
    {
        if (Input.GetKeyDown(keyY))
        {
            Debug.Log(padNumber + ": Y");
            return true;
        }
        return false;
    }

    public bool LBumberDown()
    {
        if(Input.GetKeyDown(leftBumper)) {
            return true;
        }
        return false;
    }

    public bool RBumberDown()
    {
        if (Input.GetKeyDown(rightBumper))
        {
            return true;
        }
        return false;
    }

    public float LStickHor()
    {
        if(Mathf.Abs(Input.GetAxis(leftStickX)) > safeDelta)
        {
            return Input.GetAxis(leftStickX);
        }
        return 0;
    }

    public float LStickVer()
    {
        if (Mathf.Abs(Input.GetAxis(leftStickY)) > safeDelta)
        {
            return Input.GetAxis(leftStickY);
        }
        return 0;
    }

    public float LTrigger()
    {
        if(Input.GetAxis(leftTrigger) > safeDelta)
        {
            return Input.GetAxis(leftTrigger);
        }
        return 0;
    }

    public float RTrigger()
    {

        if (Input.GetAxis(rightTrigger) > safeDelta)
        {
            return Input.GetAxis(rightTrigger);
        }
        return 0;
    }
}
