﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public class Params : MonoBehaviour
{
    private static Params me;
    public static Params Me
    {
        get
        {
            if (me == null)
                me = Resources.FindObjectsOfTypeAll<Params>().FirstOrDefault();
            return me;
        }
    }

    private void Awake()
    {
        me = this;
    }

    public int StartPlayerHealth;

    public int HPPerDefBar;

    public ArmorValueDictionary armorParams;

    public ReductionValueDictionary reductionParams;

    public WeaponAttackValueDictionary attackParams;
}


[Serializable]
public struct ArmorValue
{
    public ArmorType ArmorType;
    public int ArmorMaxHP;
}

[Serializable]
public class ArmorValueDictionary : Dictionary<ArmorType, int>, ISerializationCallbackReceiver
{
    [SerializeField]
    private List<ArmorValue> armors;

    public void OnAfterDeserialize()
    {
        OnBeforeSerialize();
    }

    public void OnBeforeSerialize()
    {
        Clear();
        if (armors == null)
            armors = new List<ArmorValue>();

        for (int i = 0; i < armors.Count; i++)
            if (!ContainsKey(armors[i].ArmorType))
                Add(armors[i].ArmorType, armors[i].ArmorMaxHP);
    }
}

[Serializable]
public struct ReductionValue
{
    public ReductionType ReductionType;
    public int Reduction;
}

[Serializable]
public class ReductionValueDictionary : Dictionary<ReductionType, int>, ISerializationCallbackReceiver
{
    [SerializeField]
    private List<ReductionValue> reductions;

    public void OnAfterDeserialize()
    {
        OnBeforeSerialize();
    }

    public void OnBeforeSerialize()
    {
        Clear();
        if (reductions == null)
            reductions = new List<ReductionValue>();

        for (int i = 0; i < reductions.Count; i++)
            if (!ContainsKey(reductions[i].ReductionType))
                Add(reductions[i].ReductionType, reductions[i].Reduction);
    }
}

[Serializable]
public struct WeaponAttackValue
{
    public WeaponType WeaponType;
    public int AttackValue;
}

[Serializable]
public class WeaponAttackValueDictionary : Dictionary<WeaponType, int>, ISerializationCallbackReceiver
{
    [SerializeField]
    private List<WeaponAttackValue> attackValues;

    public void OnAfterDeserialize()
    {
        OnBeforeSerialize();
    }

    public void OnBeforeSerialize()
    {
        Clear();
        if (attackValues == null)
            attackValues = new List<WeaponAttackValue>();

        for (int i = 0; i < attackValues.Count; i++)
            if (!ContainsKey(attackValues[i].WeaponType))
                Add(attackValues[i].WeaponType, attackValues[i].AttackValue);
    }
}

