﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using DG.Tweening;
using Assets.Scripts;
using System.Linq;

public class PickupManager : MonoBehaviour
{
    private static PickupManager instance;
    public static PickupManager Instance
    {
        get
        {
            if (instance == null) instance = FindObjectOfType<PickupManager>();
            return instance;
        }
        set
        {
            if (instance != null)
            {
                Debug.LogError("Too much PickupManager, DESTROY !!!!");
                Destroy(value);
            }
            instance = value;
        }
    }

    List<PickupBase> currentPickups = new List<PickupBase>();

    public List<GameObject> availablePickups = new List<GameObject>();

    public Transform disposingPosition;
    public float disposingTime = 1.0f;

    public List<Transform> pickupsPositions = new List<Transform>();
    List<Transform> availablePickupsPositions = new List<Transform>();

    public int pickupsToSpawn = 3;

    public GiantWaveScript giantWave;

    public UIDamageInfo dmgPrefab;
    public Transform dmgContent;
    private List<UIDamageInfo> infos = new List<UIDamageInfo>();

    public GameObject breakShellParticlePrefab;

    private void Start()
    {

        availablePickupsPositions.AddRange(pickupsPositions);
        infos = GetComponentsInChildren<UIDamageInfo>(true).ToList();
    }

    public void DisposeOfOldPickups()
    {
        giantWave.PlayWave();
        foreach (PickupBase pickup in currentPickups)
        {
            if (pickup.isPickable)
            {
                Tween tempTween = pickup.transform.DOMove(disposingPosition.position, disposingTime);
                tempTween.SetEase(Ease.InOutBack);
                tempTween.OnComplete(()=>
                {
                    pickup.gameObject.SetActive(false);
                    Destroy(pickup.gameObject);
                });
            }
        }
        currentPickups.Clear();
        availablePickupsPositions.Clear();
        availablePickupsPositions.AddRange(pickupsPositions);
    }


    public void ComeWithNewPickups()
    {
        if (availablePickupsPositions.Count <= 0) return;
        giantWave.PlayWave();
        availablePickupsPositions.Shuffle();
        int i = 0;
        for (i=0;i<pickupsToSpawn && i < availablePickupsPositions.Count; ++i)
        {
            GameObject tempObject = Instantiate(availablePickups[Random.Range(0, availablePickups.Count)], disposingPosition.position, disposingPosition.rotation);
            PickupBase pickup = tempObject.GetComponent<PickupBase>();
            Tween tempTween = pickup.transform.DOMove(availablePickupsPositions[i].position, disposingTime);
            tempTween.SetEase(Ease.InOutBack);
            currentPickups.Add(pickup);
        }
        availablePickupsPositions.RemoveRange(0, i);
        
    }

    public void removePickup(PickupBase pickup)
    {
        this.currentPickups.Remove(pickup);
        pickup.gameObject.transform.SetParent(null);
        if(pickup is Armor)
        {
            GameObject go = Instantiate(breakShellParticlePrefab, pickup.transform.position, Quaternion.identity);
            go.GetComponent<ParticleSystem>().Play();
        }
        Destroy(pickup.gameObject);

    }

    public void SpawnDmg(Transform damagedTransform, int dmg)
    {
        foreach (var player in HermitsManager.Instance.SpawnedHermits.Values.Select(x => x.GetComponent<HermitController>()))
        {
            UIDamageInfo info = GetFreeDmgInfo();
            info.Show(dmg, player.UICamera);
            info.transform.position = damagedTransform.position;
            Extensions.SetLayerRecursively(info.gameObject, 11 + player.HermitNo);
        }
    }

    public void HideDmg(UIDamageInfo info)
    {
        info.gameObject.SetActive(false);
    }

    private UIDamageInfo GetFreeDmgInfo()
    {
        foreach (var v in infos)
            if (!v.gameObject.activeSelf)
                return v;
        var go = Instantiate(dmgPrefab, dmgContent);
        infos.Add(go);
        return go;
    }

#if UNITY_EDITOR
    private void Update()
    {
        if (Input.GetKeyDown(KeyCode.Space))
        {
            if (currentPickups.Count > 0)
                DisposeOfOldPickups();
            else
                ComeWithNewPickups();
        }
    }
#endif

}
