﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using DG.Tweening;

namespace OldScripts
{
    public enum Player { one, two, three, four, none }
    public enum Team { one, two, three, four, none, invalid }

    public class PlayerScript : MonoBehaviour
    {
        public Team team;
        public Player playerNumber;
        public int maxhp = 100;

        public int hp;
        public float speed = 5f;
        public float force = 10f;

        public float speedModifier = 1f;

        [HideInInspector]
        public Vector3 lastOrietation;
        Vector3 currentOrientation;


        public Collider myCollider;
        [HideInInspector]
        public Animator myAnimator;
        [HideInInspector]
        public Transform mySpawnPoint;

        string playerControlsPrefix = "";

        string moveUpDown = "upDown";
        string moveLeftRight = "leftRight";
        string use = "use";
        string fire = "fire";
        bool moved = true;

        public bool dead = false;

        public void UpdateTeamColor()
        {
            foreach (SkinnedMeshRenderer smr in GetComponentsInChildren<SkinnedMeshRenderer>())
            {
                switch (team)
                {
                    case Team.one:
                        smr.material = OldGameManager.Instance.teamOneMaterial;
                        break;
                    case Team.two:
                        smr.material = OldGameManager.Instance.teamTwoMaterial;
                        break;
                    case Team.three:
                        smr.material = OldGameManager.Instance.teamThreeMaterial;
                        break;
                    case Team.four:
                        smr.material = OldGameManager.Instance.teamFourMaterial;
                        break;
                    case Team.none:
                        smr.material = OldGameManager.Instance.teamNoneMaterial;
                        break;
                }
            }
        }

        void Awake()
        {
            if (myCollider == null) myCollider = GetComponent<Collider>();
            myAnimator = GetComponentInChildren<Animator>();

            myAnimator.SetFloat("offset", Random.Range(0f, 1f));
            myAnimator.SetFloat("walkSpeed", speed * 0.3f);
            UpdateTeamColor();
        }

        void Start()
        {
            Reset();
            switch (playerNumber)
            {
                case Player.one:
                    playerControlsPrefix = "ONE_";
                    break;
                case Player.two:
                    playerControlsPrefix = "TWO_";
                    break;
                case Player.three:
                    playerControlsPrefix = "THREE_";
                    break;
                case Player.four:
                    playerControlsPrefix = "FOUR_";
                    break;
                default:
                    playerControlsPrefix = "NONE_";
                    break;
            }

            moveUpDown = playerControlsPrefix + moveUpDown;
            moveLeftRight = playerControlsPrefix + moveLeftRight;
            use = playerControlsPrefix + use;
            fire = playerControlsPrefix + fire;
        }

        public void Reset()
        {
            myAnimator.SetBool("Run", false);
            myAnimator.SetBool("Die", false);
            hp = maxhp;
            dead = false;
        }

        float upDownAxis = 0f, leftRightAxis = 0f;

        void Update()
        {
            if (transform.position.y < -3f)
            {
                GotHit(100);
                transform.position = OldGameManager.Instance.GetClosestRespawn(this).position;
                //transform.position = Vector3.up * 2f;
            }
            if (!dead)
            {
                upDownAxis = Input.GetAxis(moveUpDown);
                leftRightAxis = Input.GetAxis(moveLeftRight);
                if (upDownAxis > 0)
                {
                    transform.Translate(Vector3.forward * (speed * speedModifier) * Time.deltaTime, Space.World);
                    moved = true;
                    currentOrientation += Vector3.forward;
                }
                if (upDownAxis < 0)
                {
                    transform.Translate(Vector3.back * (speed * speedModifier) * Time.deltaTime, Space.World);
                    moved = true;
                    currentOrientation += Vector3.back;
                }
                if (leftRightAxis < 0)
                {
                    transform.Translate(Vector3.left * (speed * speedModifier) * Time.deltaTime, Space.World);
                    moved = true;
                    currentOrientation += Vector3.left;
                }
                if (leftRightAxis > 0)
                {
                    transform.Translate(Vector3.right * (speed * speedModifier) * Time.deltaTime, Space.World);
                    moved = true;
                    currentOrientation += Vector3.right;
                }
                myAnimator.SetBool("Run", moved);
                if (Input.GetButtonDown(use))
                {

                }
            }
        }

        public void GotHit(int healthPoints)
        {
            if (hp <= 0) return;
            hp -= healthPoints;
            if (hp <= 0)
            {
                dead = true;

                hp = 0;
                myAnimator.SetBool("Die", true);
                myAnimator.SetBool("Run", false);
            }
        }



    }
}