﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

public class UIDefenseBars : MonoBehaviour
{
    [SerializeField]
    private UIShieldBar miniSliderPrefab;

    [SerializeField]
    private RectTransform slidersContent;

    [SerializeField]
    private List<UIShieldBar> spawnedSliders = new List<UIShieldBar>();

    private List<UIShieldBar> activeSliders = new List<UIShieldBar>();

    [SerializeField]
    private Sprite normalFull, brokenFull, normalBack, brokenBack;

    [SerializeField]
    private Image shieldImage;

    [SerializeField]
    private Sprite shFull, shHalf, shNone;

    private Armor armor;

    private int lastHp, lastActiveBar, maxHP;


    public void Show()
    {
        gameObject.SetActive(true);
    }

    public void Hide()
    {
        gameObject.SetActive(false);
    }

    public void Reinit(Armor armor)
    {
        this.armor = armor;
        if (armor == null)
        {
            ReadySliders(0);
            shieldImage.sprite = shNone;
            return;
        }
        maxHP = Params.Me.armorParams[armor.armorType];
        lastHp = (int) armor.GetRemainingHPValue();
        lastActiveBar = lastHp / Params.Me.HPPerDefBar;
        ReadySliders(maxHP / Params.Me.HPPerDefBar);
        shieldImage.sprite = shFull;
        Refresh();
    }

    public void Refresh()
    {
        if (armor == null)
            return;

        float currentHp = armor.GetRemainingHPValue();
        if(currentHp / Params.Me.HPPerDefBar < lastHp / Params.Me.HPPerDefBar)
        {
            // destroy last bar animation
        }

        for (int i = 0; i < activeSliders.Count; i++)
        {
            activeSliders[i].SetValue(currentHp / Params.Me.HPPerDefBar - i);
        }

        lastHp = (int)currentHp;
        lastActiveBar = (int)Mathf.Ceil(lastHp / Params.Me.HPPerDefBar);

        if ((float)lastHp / maxHP < 0.5f)
            shieldImage.sprite = shHalf;

    }

    private void ReadySliders(int count)
    {
        foreach (var v in activeSliders)
            v.gameObject.SetActive(false);
        activeSliders.Clear();
        while (activeSliders.Count < count)
        {
            activeSliders.Add(GetNewSlider());
        }
        for (int i = 0; i < activeSliders.Count; i++)
        {
            if (i < activeSliders.Count / 2)
                activeSliders[i].SetSprite(brokenBack, brokenFull);
            else
                activeSliders[i].SetSprite(normalBack, normalFull);
        }
    }

    private UIShieldBar GetNewSlider()
    {
        foreach (var v in spawnedSliders)
            if (!v.gameObject.activeSelf)
            {
                v.gameObject.SetActive(true);
                return v;
            }

        return Instantiate(miniSliderPrefab, slidersContent);
    }


}
